import base64
import configparser
import os
import subprocess

from charmhelpers import fetch
from charmhelpers.core import (
    hookenv,
    host,
)
from charmhelpers.core.services import helpers
from charmhelpers.payload import (archive, execd)

from charmhelpers.contrib.charmsupport import nrpe


REQUIRED_PACKAGES = [
    'python-virtualenv', 'python3-dev', 'python3-jinja2'
]

WSGI_USER = 'www-data'
WSGI_GROUP = 'www-data'

config = hookenv.config()

BASE_DIR = '/srv/result-enum-api/{}'.format(config['environment'])
SERVICE_DIR = os.path.join(BASE_DIR, 'result-enum-api')
LOG_DIR = os.path.join(BASE_DIR, 'logs')


def log_start(service_name):
    hookenv.log('result-enum-api starting')


def ensure_directories(service_name):
    host.mkdir(BASE_DIR, owner='root', group='ubuntu')


def basenode(service_name):
    hookenv.log("Executing basenode")
    execd.execd_preinstall()


def install_packages(service_name):
    hookenv.log('Installing dependencies...')
    fetch.configure_sources(update=True)
    fetch.apt_install(REQUIRED_PACKAGES, fatal=True)


def get_cloud_service_from_tarball(service_name):
    files_dir = os.path.join(hookenv.charm_dir(), 'files')
    tarball = os.path.join(files_dir, 'result-enum-api.tgz')
    if not os.path.exists(SERVICE_DIR):
        hookenv.log('Installing the code for the first time from tarball')
        archive.extract_tarfile(tarball, os.path.dirname(SERVICE_DIR))


def get_config_file(service_name):
    config = hookenv.config()
    config_content = base64.b64decode(config['config-file']).decode()
    config_path = os.path.join(SERVICE_DIR, 'result-enum-api.conf')
    parser = configparser.ConfigParser()
    parser.read_string(config_content)
    mongo_addresses = []
    relations = hookenv.relations_of_type('mongodb')
    for rel in relations:
        host = rel['private-address']
        port = rel.get('port', None)
        if port:
            mongo_address = '%s:%s' % (host, port)
        else:
            mongo_address = host
        mongo_addresses.append(mongo_address)
    assert mongo_addresses != []
    parser['mongodb']['server'] = ','.join(mongo_addresses)
    with open(config_path, 'w') as f:
        parser.write(f)


def install_python_packages(service_name):
    env_dir = os.path.join(SERVICE_DIR, 've')
    if os.path.exists(env_dir):
        hookenv.log('Service venv already exists, nothing to do ...')
        return

    hookenv.log('Installing python packages into the venv ...')
    subprocess.check_call(['virtualenv', '-p', 'python3', env_dir])

    pip_cache = os.path.join(hookenv.charm_dir(), 'files', 'pip-cache')
    requirements = os.path.join(SERVICE_DIR, 'requirements.txt')

    hookenv.log('Installing from download cache.')
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install',
                           '--no-index',
                           '--find-links={}'.format(pip_cache),
                           '-r', requirements])

    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install', '--no-deps', '-e', SERVICE_DIR])


def prepare_for_wsgi(service_name):
    host.adduser(WSGI_USER)
    host.add_group(WSGI_GROUP)
    host.add_user_to_group(WSGI_USER, WSGI_GROUP)
    if not os.path.exists(LOG_DIR):
        host.mkdir(LOG_DIR, WSGI_USER, WSGI_GROUP, perms=0o755)


def publish_website_relation_data(service_name):
    # Due to Bug #1409763, this functionality is as action rather than the
    # provided_data below.
    for relid in hookenv.relation_ids('website'):
        hookenv.log(30 * '*')
        hookenv.log('Hack for #1409763: {}'.format(relid))
        hookenv.log(30 * '*')
        hookenv.relation_set(
            relid, host=hookenv.unit_get('private-address'), port=8000)


class WebsiteRelation(helpers.HttpRelation):

    def provide_data(self):
        hookenv.log('Providing Website relation data.')
        data = super(WebsiteRelation, self).provide_data()
        data['port'] = 8000
        return data


def nrpe_external_master_relation(service_name):
    ''' Configure the nrpe-external-master relation '''
    nrpe_compat = nrpe.NRPE()

    nrpe_compat.add_check(
        shortname="check_app_http",
        description="Check app http",
        check_cmd="/usr/lib/nagios/plugins/check_http -I 127.0.0.1 -p 8000 -e ' 200 OK' -u '/'",
        )

    nrpe_compat.write()
